/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2011 Tirasa. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 */
package org.connid.ad.util;

import java.io.UnsupportedEncodingException;

import java.util.List;
import javax.naming.directory.BasicAttribute;

import org.identityconnectors.common.logging.Log;
import org.identityconnectors.common.security.GuardedString;
import org.identityconnectors.framework.common.objects.Attribute;
import org.identityconnectors.framework.common.objects.OperationalAttributes;

public abstract class ADGuardedPasswordAttribute {

    private static final Log LOG = Log.getLog(ADGuardedPasswordAttribute.class);

    public static ADGuardedPasswordAttribute create(
            final String attrName, final Attribute attr) {

        assert attr.is(OperationalAttributes.PASSWORD_NAME);

        final List<Object> value = attr.getValue();

        if (value != null && !value.isEmpty()) {
            return ADGuardedPasswordAttribute.create(attrName,
                    (GuardedString) value.get(0));
        } else {
            return ADGuardedPasswordAttribute.create(attrName);
        }
    }

    public static ADGuardedPasswordAttribute create(
            final String attrName, final GuardedString password) {
        return new Simple(attrName, password);
    }

    public static ADGuardedPasswordAttribute create(final String attrName) {
        return new Empty(attrName);
    }

    public abstract void access(final Accessor accessor);

    public interface Accessor {

        void access(BasicAttribute passwordAttribute);
    }

    private static final class Simple extends ADGuardedPasswordAttribute {

        private final String attrName;

        private final GuardedString password;

        private Simple(String attrName, GuardedString password) {
            this.attrName = attrName;
            this.password = password;
        }

        @Override
        public void access(final Accessor accessor) {
            password.access(new GuardedString.Accessor() {

                @Override
                public void access(char[] clearChars) {
                    final String quotedPwd = "\"" + new String(clearChars) + "\"";

                    try {

                        byte[] unicodePwd = quotedPwd.getBytes("UTF-16LE");

                        final BasicAttribute attr =
                                new BasicAttribute(attrName, unicodePwd);

                        accessor.access(attr);

                    } catch (UnsupportedEncodingException e) {
                        LOG.error(e, "Error encoding password");
                    }
                }
            });
        }
    }

    private static final class Empty extends ADGuardedPasswordAttribute {

        private final String attrName;

        private Empty(String attrName) {
            this.attrName = attrName;
        }

        @Override
        public void access(Accessor accessor) {
            accessor.access(new BasicAttribute(attrName));
        }
    }
}